#include "stdafx.h"
#include "TVTest.h"
#include "AppMain.h"
#include "ColorScheme.h"
#include "Settings.h"
#include "DialogUtil.h"
#include "DrawUtil.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


static const LPCTSTR GradientDirectionList[] = {
	TEXT("horizontal"),
	TEXT("vertical"),
	TEXT("horizontal-mirror"),
	TEXT("vertical-mirror"),
};


#define HEXRGB(hex) RGB((hex)>>16,((hex)>>8)&0xFF,(hex)&0xFF)

const CColorScheme::ColorInfo CColorScheme::m_ColorInfoList[NUM_COLORS] = {
	{HEXRGB(0x777777),	TEXT("StatusBack"),							TEXT("Xe[^Xo[wi1")},
	{HEXRGB(0x222222),	TEXT("StatusBack2"),						TEXT("Xe[^Xo[wi2")},
	{HEXRGB(0xBBBBBB),	TEXT("StatusText"),							TEXT("Xe[^Xo[")},
	{HEXRGB(0x777777),	TEXT("StatusItemBorder"),					TEXT("Xe[^Xo[ڊOg")},
	{HEXRGB(0x222222),	TEXT("StatusBottomItemBack"),				TEXT("Xe[^Xo[iwi1")},
	{HEXRGB(0x222222),	TEXT("StatusBottomItemBack2"),				TEXT("Xe[^Xo[iwi2")},
	{HEXRGB(0xBBBBBB),	TEXT("StatusBottomItemText"),				TEXT("Xe[^Xo[i")},
	{HEXRGB(0x222222),	TEXT("StatusBottomItemBorder"),				TEXT("Xe[^Xo[iOg")},
	{HEXRGB(0x3384FF),	TEXT("StatusHighlightBack"),				TEXT("Xe[^Xo[Iwi1")},
	{HEXRGB(0x33D6FF),	TEXT("StatusHighlightBack2"),				TEXT("Xe[^Xo[Iwi2")},
	{HEXRGB(0x333333),	TEXT("StatusHighlightText"),				TEXT("Xe[^Xo[I𕶎")},
	{HEXRGB(0x3384FF),	TEXT("StatusHighlightBorder"),				TEXT("Xe[^Xo[IOg")},
	{HEXRGB(0x777777),	TEXT("StatusBorder"),						TEXT("Xe[^Xo[Og")},
	{HEXRGB(0xDF3F00),	TEXT("StatusRecordingCircle"),				TEXT("Xe[^Xo[^恜")},
	{HEXRGB(0xCCCCCC),	TEXT("Splitter"),							TEXT("")},
	{HEXRGB(0x777777),	TEXT("ScreenBorder"),						TEXT("ʂ̊Og")},
	{HEXRGB(0x666666),	TEXT("PanelBack"),							TEXT("plwi")},
	{HEXRGB(0xCCCCCC),	TEXT("PanelText"),							TEXT("pl")},
	{HEXRGB(0xCCCCCC),	TEXT("PanelTabBack"),						TEXT("pl^uwi1")},
	{HEXRGB(0x888888),	TEXT("PanelTabBack2"),						TEXT("pl^uwi2")},
	{HEXRGB(0x444444),	TEXT("PanelTabText"),						TEXT("pl^u")},
	{HEXRGB(0x444444),	TEXT("PanelTabBorder"),						TEXT("pl^uOg")},
	{HEXRGB(0x999999),	TEXT("PanelCurTabBack"),					TEXT("plI^uwi1")},
	{HEXRGB(0x666666),	TEXT("PanelCurTabBack2"),					TEXT("plI^uwi2")},
	{HEXRGB(0xEEEEEE),	TEXT("PanelCurTabText"),					TEXT("plI^u")},
	{HEXRGB(0x444444),	TEXT("PanelCurTabBorder"),					TEXT("plI^uOg")},
	{HEXRGB(0x888888),	TEXT("PanelTabMargin"),						TEXT("pl^u]1")},
	{HEXRGB(0x888888),	TEXT("PanelTabMargin2"),					TEXT("pl^u]2")},
	{HEXRGB(0x888888),	TEXT("PanelTabMarginBorder"),				TEXT("pl^u]Og")},
	{HEXRGB(0x444444),	TEXT("PanelTabLine"),						TEXT("pl^u")},
	{HEXRGB(0x777777),	TEXT("PanelTitleBack"),						TEXT("pl^Cgwi1")},
	{HEXRGB(0x222222),	TEXT("PanelTitleBack2"),					TEXT("pl^Cgwi2")},
	{HEXRGB(0xBBBBBB),	TEXT("PanelTitleText"),						TEXT("pl^Cg")},
	{HEXRGB(0x777777),	TEXT("PanelTitleBorder"),					TEXT("pl^CgOg")},
	{HEXRGB(0x777777),	TEXT("ProgramInfoBack"),					TEXT("plԑgwi")},
	{HEXRGB(0xDDDDDD),	TEXT("ProgramInfoText"),					TEXT("plԑg")},
	{HEXRGB(0x777777),	TEXT("ProgramListBack"),					TEXT("ԑg\plԑgewi1")},
	{HEXRGB(0x888888),	TEXT("ProgramListBack2"),					TEXT("ԑg\plԑgewi2")},
	{HEXRGB(0xDDDDDD),	TEXT("ProgramListText"),					TEXT("ԑg\plԑge")},
	{HEXRGB(0x777777),	TEXT("ProgramListBorder"),					TEXT("ԑg\plԑgeOg")},
	{HEXRGB(0x777777),	TEXT("ProgramListCurBack"),					TEXT("ԑg\plݔԑgewi1")},
	{HEXRGB(0x888888),	TEXT("ProgramListCurBack2"),				TEXT("ԑg\plݔԑgewi2")},
	{HEXRGB(0xDDDDDD),	TEXT("ProgramListCurText"),					TEXT("ԑg\plݔԑge")},
	{HEXRGB(0x777777),	TEXT("ProgramListCurBorder"),				TEXT("ԑg\plݔԑgeOg")},
	{HEXRGB(0x777777),	TEXT("ProgramListTitleBack"),				TEXT("ԑg\plԑgwi1")},
	{HEXRGB(0x555555),	TEXT("ProgramListTitleBack2"),				TEXT("ԑg\plԑgwi2")},
	{HEXRGB(0xCCCCCC),	TEXT("ProgramListTitleText"),				TEXT("ԑg\plԑg")},
	{HEXRGB(0x777777),	TEXT("ProgramListTitleBorder"),				TEXT("ԑg\plԑgOg")},
	{HEXRGB(0x3384FF),	TEXT("ProgramListCurTitleBack"),			TEXT("ԑg\plݔԑgwi1")},
	{HEXRGB(0x33D6FF),	TEXT("ProgramListCurTitleBack2"),			TEXT("ԑg\plݔԑgwi2")},
	{HEXRGB(0x333333),	TEXT("ProgramListCurTitleText"),			TEXT("ԑg\plݔԑg")},
	{HEXRGB(0x3384FF),	TEXT("ProgramListCurTitleBorder"),			TEXT("ԑg\plݔԑgOg")},
	{HEXRGB(0x777777),	TEXT("ChannelPanelChannelNameBack"),		TEXT("`lplǖwi1")},
	{HEXRGB(0x555555),	TEXT("ChannelPanelChannelNameBack2"),		TEXT("`lplǖwi2")},
	{HEXRGB(0xCCCCCC),	TEXT("ChannelPanelChannelNameText"),		TEXT("`lplǖ")},
	{HEXRGB(0x555555),	TEXT("ChannelPanelChannelNameBorder"),		TEXT("`lplǖOg")},
	{HEXRGB(0x3384FF),	TEXT("ChannelPanelCurChannelNameBack"),		TEXT("`lpl݋ǖwi1")},
	{HEXRGB(0x33D6FF),	TEXT("ChannelPanelCurChannelNameBack2"),	TEXT("`lpl݋ǖwi2")},
	{HEXRGB(0x333333),	TEXT("ChannelPanelCurChannelNameText"),		TEXT("`lpl݋ǖ")},
	{HEXRGB(0x3384FF),	TEXT("ChannelPanelCurChannelNameBorder"),	TEXT("`lpl݋ǖOg")},
	{HEXRGB(0x777777),	TEXT("ChannelPanelEventNameBack"),			TEXT("`lplԑg1wi1")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelEventNameBack2"),			TEXT("`lplԑg1wi2")},
	{HEXRGB(0xDDDDDD),	TEXT("ChannelPanelEventNameText"),			TEXT("`lplԑg1")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelEventNameBorder"),		TEXT("`lplԑg1Og")},
	{HEXRGB(0x777777),	TEXT("ChannelPanelEventName2Back"),			TEXT("`lplԑg2wi1")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelEventName2Back2"),		TEXT("`lplԑg2wi2")},
	{HEXRGB(0xDDDDDD),	TEXT("ChannelPanelEventName2Text"),			TEXT("`lplԑg2")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelEventName2Border"),		TEXT("`lplԑg2Og")},
	{HEXRGB(0x777777),	TEXT("ChannelPanelCurEventNameBack"),		TEXT("`lplIԑg1wi1")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelCurEventNameBack2"),		TEXT("`lplIԑg1wi2")},
	{HEXRGB(0xDDDDDD),	TEXT("ChannelPanelCurEventNameText"),		TEXT("`lplIԑg1")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelCurEventNameBorder"),		TEXT("`lplIԑg1Og")},
	{HEXRGB(0x777777),	TEXT("ChannelPanelCurEventName2Back"),		TEXT("`lplIԑg2wi1")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelCurEventName2Back2"),		TEXT("`lplIԑg2wi2")},
	{HEXRGB(0xDDDDDD),	TEXT("ChannelPanelCurEventName2Text"),		TEXT("`lplIԑg2")},
	{HEXRGB(0x888888),	TEXT("ChannelPanelCurEventName2Border"),	TEXT("`lplIԑg2Og")},
	{HEXRGB(0x666666),	TEXT("ControlPanelBack"),					TEXT("plwi1")},
	{HEXRGB(0x666666),	TEXT("ControlPanelBack2"),					TEXT("plwi2")},
	{HEXRGB(0xCCCCCC),	TEXT("ControlPanelText"),					TEXT("pl")},
	{HEXRGB(0x666666),	TEXT("ControlPanelItemBorder"),				TEXT("plڊOg")},
	{HEXRGB(0x3384FF),	TEXT("ControlPanelHighlightBack"),			TEXT("plIwi1")},
	{HEXRGB(0x33D6FF),	TEXT("ControlPanelHighlightBack2"),			TEXT("plIwi2")},
	{HEXRGB(0xEEEEEE),	TEXT("ControlPanelHighlightText"),			TEXT("plI𕶎")},
	{HEXRGB(0x3384FF),	TEXT("ControlPanelHighlightBorder"),		TEXT("plIڊOg")},
	{HEXRGB(0x666666),	TEXT("ControlPanelMargin"),					TEXT("pl]")},
	{HEXRGB(0x777777),	TEXT("CaptionPanelBack"),					TEXT("plwi")},
	{HEXRGB(0xDDDDDD),	TEXT("CaptionPanelText"),					TEXT("pl")},
	{HEXRGB(0x777777),	TEXT("TitleBarBack"),						TEXT("^Cgo[wi1")},
	{HEXRGB(0x222222),	TEXT("TitleBarBack2"),						TEXT("^Cgo[wi2")},
	{HEXRGB(0xBBBBBB),	TEXT("TitleBarText"),						TEXT("^Cgo[")},
	{HEXRGB(0x777777),	TEXT("TitleBarTextBorder"),					TEXT("^Cgo[Og")},
	{HEXRGB(0x777777),	TEXT("TitleBarIconBack"),					TEXT("^Cgo[ACRwi1")},
	{HEXRGB(0x222222),	TEXT("TitleBarIconBack2"),					TEXT("^Cgo[ACRwi2")},
	{HEXRGB(0xBBBBBB),	TEXT("TitleBarIcon"),						TEXT("^Cgo[ACR")},
	{HEXRGB(0x777777),	TEXT("TitleBarIconBorder"),					TEXT("^Cgo[ACROg")},
	{HEXRGB(0x3384FF),	TEXT("TitleBarHighlightBack"),				TEXT("^Cgo[Iwi1")},
	{HEXRGB(0x33D6FF),	TEXT("TitleBarHighlightBack2"),				TEXT("^Cgo[Iwi2")},
	{HEXRGB(0x444444),	TEXT("TitleBarHighlightIcon"),				TEXT("^Cgo[IACR")},
	{HEXRGB(0x3384FF),	TEXT("TitleBarHighlightIconBorder"),		TEXT("^Cgo[IACROg")},
	{HEXRGB(0x777777),	TEXT("TitleBarBorder"),						TEXT("^Cgo[Og")},
	{HEXRGB(0x777777),	TEXT("SideBarBack"),						TEXT("TCho[wi1")},
	{HEXRGB(0x222222),	TEXT("SideBarBack2"),						TEXT("TCho[wi2")},
	{HEXRGB(0xBBBBBB),	TEXT("SideBarIcon"),						TEXT("TCho[ACR")},
	{HEXRGB(0x777777),	TEXT("SideBarItemBorder"),					TEXT("TCho[ڊOg")},
	{HEXRGB(0x3384FF),	TEXT("SideBarHighlightBack"),				TEXT("TCho[Iwi1")},
	{HEXRGB(0x33D6FF),	TEXT("SideBarHighlightBack2"),				TEXT("TCho[Iwi2")},
	{HEXRGB(0x444444),	TEXT("SideBarHighlightIcon"),				TEXT("TCho[IACR")},
	{HEXRGB(0x3384FF),	TEXT("SideBarHighlightBorder"),				TEXT("TCho[IOg")},
	{HEXRGB(0x777777),	TEXT("SideBarCheckBack"),					TEXT("TCho[`FbNwi1")},
	{HEXRGB(0x222222),	TEXT("SideBarCheckBack2"),					TEXT("TCho[`FbNwi2")},
	{HEXRGB(0xBBBBBB),	TEXT("SideBarCheckIcon"),					TEXT("TCho[`FbNACR")},
	{HEXRGB(0x222222),	TEXT("SideBarCheckBorder"),					TEXT("TCho[`FbNOg")},
	{HEXRGB(0x777777),	TEXT("SideBarBorder"),						TEXT("TCho[Og")},
	{HEXRGB(0x222222),	TEXT("NotificationBarBack"),				TEXT("ʒmo[wi1")},
	{HEXRGB(0x333333),	TEXT("NotificationBarBack2"),				TEXT("ʒmo[wi2")},
	{HEXRGB(0xBBBBBB),	TEXT("NotificationBarText"),				TEXT("ʒmo[")},
	{HEXRGB(0xFF9F44),	TEXT("NotificationBarWarningText"),			TEXT("ʒmo[x")},
	{HEXRGB(0xFF4444),	TEXT("NotificationBarErrorText"),			TEXT("ʒmo[G[")},
	{HEXRGB(0x666666),	TEXT("ProgramGuideBack"),					TEXT("EPGԑg\wi")},
	{HEXRGB(0x000000),	TEXT("ProgramGuideText"),					TEXT("EPGԑg\")},
	{HEXRGB(0x777777),	TEXT("ProgramGuideChannelBack"),			TEXT("EPGԑg\`lwi1")},
	{HEXRGB(0x222222),	TEXT("ProgramGuideChannelBack2"),			TEXT("EPGԑg\`lwi2")},
	{HEXRGB(0xBBBBBB),	TEXT("ProgramGuideChannelText"),			TEXT("EPGԑg\`l")},
	{HEXRGB(0x3384FF),	TEXT("ProgramGuideCurChannelBack"),			TEXT("EPGԑg\`lIwi1")},
	{HEXRGB(0x33D6FF),	TEXT("ProgramGuideCurChannelBack2"),		TEXT("EPGԑg\`lIwi2")},
	{HEXRGB(0x333333),	TEXT("ProgramGuideCurChannelText"),			TEXT("EPGԑg\`lI𕶎")},
	{HEXRGB(0x888888),	TEXT("ProgramGuideTimeBack"),				TEXT("EPGԑg\wi1")},
	{HEXRGB(0x777777),	TEXT("ProgramGuideTimeBack2"),				TEXT("EPGԑg\wi2")},
	{HEXRGB(0x004CBF),	TEXT("ProgramGuideTime0To2Back"),			TEXT("EPGԑg\0`2wi1")},
	{HEXRGB(0x00337F),	TEXT("ProgramGuideTime0To2Back2"),			TEXT("EPGԑg\0`2wi2")},
	{HEXRGB(0x0099BF),	TEXT("ProgramGuideTime3To5Back"),			TEXT("EPGԑg\3`5wi1")},
	{HEXRGB(0x00667F),	TEXT("ProgramGuideTime3To5Back2"),			TEXT("EPGԑg\3`5wi2")},
	{HEXRGB(0x00BF99),	TEXT("ProgramGuideTime6To8Back"),			TEXT("EPGԑg\6`8wi1")},
	{HEXRGB(0x007F66),	TEXT("ProgramGuideTime6To8Back2"),			TEXT("EPGԑg\6`8wi2")},
	{HEXRGB(0x99BF00),	TEXT("ProgramGuideTime9To11Back"),			TEXT("EPGԑg\9`11wi1")},
	{HEXRGB(0x667F00),	TEXT("ProgramGuideTime9To11Back2"),			TEXT("EPGԑg\9`11wi2")},
	{HEXRGB(0xBF9900),	TEXT("ProgramGuideTime12To14Back"),			TEXT("EPGԑg\12`14wi1")},
	{HEXRGB(0x7F6600),	TEXT("ProgramGuideTime12To14Back2"),		TEXT("EPGԑg\12`14wi2")},
	{HEXRGB(0xBF4C00),	TEXT("ProgramGuideTime15To17Back"),			TEXT("EPGԑg\15`17wi1")},
	{HEXRGB(0x7F3300),	TEXT("ProgramGuideTime15To17Back2"),		TEXT("EPGԑg\15`17Ԕwi2")},
	{HEXRGB(0xBF0099),	TEXT("ProgramGuideTime18To20Back"),			TEXT("EPGԑg\18`20wi1")},
	{HEXRGB(0x7F0066),	TEXT("ProgramGuideTime18To20Back2"),		TEXT("EPGԑg\18`20wi2")},
	{HEXRGB(0x9900BF),	TEXT("ProgramGuideTime21To23Back"),			TEXT("EPGԑg\21`23wi1")},
	{HEXRGB(0x66007F),	TEXT("ProgramGuideTime21To23Back2"),		TEXT("EPGԑg\21`23wi2")},
	{HEXRGB(0xDDDDDD),	TEXT("ProgramGuideTimeText"),				TEXT("EPGԑg\ԕ")},
	{HEXRGB(0xBBBBBB),	TEXT("ProgramGuideTimeLine"),				TEXT("EPGԑg\Ԑ")},
	{HEXRGB(0xFF6600),	TEXT("ProgramGuideCurTimeLine"),			TEXT("EPGԑg\ݎ")},
	{RGB(255,255,224),	TEXT("EPGContentNews"),						TEXT("EPGԑg\j[Xԑg")},
	{RGB(224,224,255),	TEXT("EPGContentSports"),					TEXT("EPGԑg\X|[cԑg")},
	{RGB(255,224,240),	TEXT("EPGContentInformation"),				TEXT("EPGԑg\ԑg")},
	{RGB(255,224,224),	TEXT("EPGContentDrama"),					TEXT("EPGԑg\h}")},
	{RGB(224,255,224),	TEXT("EPGContentMusic"),					TEXT("EPGԑg\yԑg")},
	{RGB(224,255,255),	TEXT("EPGContentVariety"),					TEXT("EPGԑg\oGeBԑg")},
	{RGB(255,240,224),	TEXT("EPGContentMovie"),					TEXT("EPGԑg\f")},
	{RGB(255,224,255),	TEXT("EPGContentAnime"),					TEXT("EPGԑg\Aj/B")},
	{RGB(255,255,224),	TEXT("EPGContentDocumentary"),				TEXT("EPGԑg\hL^[/{ԑg")},
	{RGB(255,240,224),	TEXT("EPGContentTheater"),					TEXT("EPGԑg\/")},
	{RGB(224,240,255),	TEXT("EPGContentEducation"),				TEXT("EPGԑg\/ԑg")},
	{RGB(224,240,255),	TEXT("EPGContentWelfare"),					TEXT("EPGԑg\ԑg")},
	{RGB(240,240,240),	TEXT("EPGContentOther"),					TEXT("EPGԑg\̑̔ԑg")},
};

const CColorScheme::GradientInfo CColorScheme::m_GradientInfoList[NUM_GRADIENTS] = {
	{TEXT("StatusBackGradient"),						Theme::DIRECTION_VERT,	false,
		COLOR_STATUSBACK1,						COLOR_STATUSBACK2},
	{TEXT("StatusBottomItemBackGradient"),				Theme::DIRECTION_VERT,	false,
		COLOR_STATUSBOTTOMITEMBACK1,			COLOR_STATUSBOTTOMITEMBACK2},
	{TEXT("StatusHighlightBackGradient"),				Theme::DIRECTION_VERT,	true,
		COLOR_STATUSHIGHLIGHTBACK1,				COLOR_STATUSHIGHLIGHTBACK2},
	{TEXT("PanelTabBackGradient"),						Theme::DIRECTION_VERT,	true,
		COLOR_PANELTABBACK1,					COLOR_PANELTABBACK2},
	{TEXT("PanelCurTabBackGradient"),					Theme::DIRECTION_VERT,	true,
		COLOR_PANELCURTABBACK1,					COLOR_PANELCURTABBACK2},
	{TEXT("PanelTabMarginGradient"),					Theme::DIRECTION_VERT,	false,
		COLOR_PANELTABMARGIN1,					COLOR_PANELTABMARGIN2},
	{TEXT("PanelTitleBackGradient"),					Theme::DIRECTION_VERT,	true,
		COLOR_PANELTITLEBACK1,					COLOR_PANELTITLEBACK2},
	{TEXT("ProgramListBackGradient"),					Theme::DIRECTION_VERT,	true,
		COLOR_PROGRAMLISTPANEL_EVENTBACK1,		COLOR_PROGRAMLISTPANEL_EVENTBACK2},
	{TEXT("ProgramListCurBackGradient"),				Theme::DIRECTION_VERT,	true,
		COLOR_PROGRAMLISTPANEL_CUREVENTBACK1,	COLOR_PROGRAMLISTPANEL_CUREVENTBACK2},
	{TEXT("ProgramListTitleBackGradient"),				Theme::DIRECTION_VERT,	true,
		COLOR_PROGRAMLISTPANEL_TITLEBACK1,		COLOR_PROGRAMLISTPANEL_TITLEBACK2},
	{TEXT("ProgramListCurTitleBackGradient"),			Theme::DIRECTION_VERT,	true,
		COLOR_PROGRAMLISTPANEL_CURTITLEBACK1,	COLOR_PROGRAMLISTPANEL_CURTITLEBACK2},
	{TEXT("ChannelPanelChannelNameBackGradient"),		Theme::DIRECTION_VERT,	true,
		COLOR_CHANNELPANEL_CHANNELNAMEBACK1,	COLOR_CHANNELPANEL_CHANNELNAMEBACK2},
	{TEXT("ChannelPanelCurChannelNameBackGradient"),	Theme::DIRECTION_VERT,	true,
		COLOR_CHANNELPANEL_CURCHANNELNAMEBACK1,	COLOR_CHANNELPANEL_CURCHANNELNAMEBACK2},
	{TEXT("ChannelPanelEventNameBackGradient"),			Theme::DIRECTION_VERT,	true,
		COLOR_CHANNELPANEL_EVENTNAME1BACK1,		COLOR_CHANNELPANEL_EVENTNAME1BACK2},
	{TEXT("ChannelPanelEventName2BackGradient"),		Theme::DIRECTION_VERT,	true,
		COLOR_CHANNELPANEL_EVENTNAME2BACK1,		COLOR_CHANNELPANEL_EVENTNAME2BACK2},
	{TEXT("ChannelPanelCurEventNameBackGradient"),		Theme::DIRECTION_VERT,	true,
		COLOR_CHANNELPANEL_CUREVENTNAME1BACK1,	COLOR_CHANNELPANEL_CUREVENTNAME1BACK2},
	{TEXT("ChannelPanelCurEventName2BackGradient"),		Theme::DIRECTION_VERT,	true,
		COLOR_CHANNELPANEL_CUREVENTNAME2BACK1,	COLOR_CHANNELPANEL_CUREVENTNAME2BACK2},
	{TEXT("ControlPanelBackGradient"),					Theme::DIRECTION_VERT,	true,
		COLOR_CONTROLPANELBACK1,				COLOR_CONTROLPANELBACK2},
	{TEXT("ControlPanelHighlightBackGradient"),			Theme::DIRECTION_VERT,	true,
		COLOR_CONTROLPANELHIGHLIGHTBACK1,		COLOR_CONTROLPANELHIGHLIGHTBACK2},
	{TEXT("TitleBarBackGradient"),						Theme::DIRECTION_VERT,	true,
		COLOR_TITLEBARBACK1,					COLOR_TITLEBARBACK2},
	{TEXT("TitleBarIconBackGradient"),					Theme::DIRECTION_VERT,	true,
		COLOR_TITLEBARICONBACK1,				COLOR_TITLEBARICONBACK2},
	{TEXT("TitleBarHighlightBackGradient"),				Theme::DIRECTION_VERT,	true,
		COLOR_TITLEBARHIGHLIGHTBACK1,			COLOR_TITLEBARHIGHLIGHTBACK2},
	{TEXT("SideBarBackGradient"),						Theme::DIRECTION_HORZ,	true,
		COLOR_SIDEBARBACK1,						COLOR_SIDEBARBACK2},
	{TEXT("SideBarHighlightBackGradient"),				Theme::DIRECTION_HORZ,	true,
		COLOR_SIDEBARHIGHLIGHTBACK1,			COLOR_SIDEBARHIGHLIGHTBACK2},
	{TEXT("SideBarCheckBackGradient"),					Theme::DIRECTION_HORZ,	true,
		COLOR_SIDEBARCHECKBACK1,				COLOR_SIDEBARCHECKBACK2},
	{TEXT("NotificationBarBackGradient"),				Theme::DIRECTION_VERT,	true,
		COLOR_NOTIFICATIONBARBACK1,				COLOR_NOTIFICATIONBARBACK2},
	{TEXT("ProgramGuideChannelBackGradient"),			Theme::DIRECTION_VERT,	true,
		COLOR_PROGRAMGUIDECHANNELBACK1,			COLOR_PROGRAMGUIDECHANNELBACK2},
	{TEXT("ProgramGuideCurChannelBackGradient"),		Theme::DIRECTION_VERT,	true,
		COLOR_PROGRAMGUIDECURCHANNELBACK1,		COLOR_PROGRAMGUIDECURCHANNELBACK2},
	{TEXT("ProgramGuideTimeBackGradient"),				Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK1,			COLOR_PROGRAMGUIDETIMEBACK2},
	{TEXT("ProgramGuideTime0To2BackGradient"),			Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_0TO2_1,		COLOR_PROGRAMGUIDETIMEBACK_0TO2_2},
	{TEXT("ProgramGuideTime3To5BackGradient"),			Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_3TO5_1,		COLOR_PROGRAMGUIDETIMEBACK_3TO5_2},
	{TEXT("ProgramGuideTime6To8BackGradient"),			Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_6TO8_1,		COLOR_PROGRAMGUIDETIMEBACK_6TO8_2},
	{TEXT("ProgramGuideTime9To11BackGradient"),			Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_9TO11_1,		COLOR_PROGRAMGUIDETIMEBACK_9TO11_2},
	{TEXT("ProgramGuideTime12To14BackGradient"),		Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_12TO14_1,	COLOR_PROGRAMGUIDETIMEBACK_12TO14_2},
	{TEXT("ProgramGuideTime15To17BackGradient"),		Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_15TO17_1,	COLOR_PROGRAMGUIDETIMEBACK_15TO17_2},
	{TEXT("ProgramGuideTime18To20BackGradient"),		Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_18TO20_1,	COLOR_PROGRAMGUIDETIMEBACK_18TO20_2},
	{TEXT("ProgramGuideTime21To23BackGradient"),		Theme::DIRECTION_HORZ,	true,
		COLOR_PROGRAMGUIDETIMEBACK_21TO23_1,	COLOR_PROGRAMGUIDETIMEBACK_21TO23_2},
};

const CColorScheme::BorderInfo CColorScheme::m_BorderInfoList[NUM_BORDERS] = {
	{TEXT("ScreenBorder"),						Theme::BORDER_SUNKEN,
		COLOR_SCREENBORDER,							true},
	{TEXT("StatusBorder"),						Theme::BORDER_RAISED,
		COLOR_STATUSBORDER,							false},
	{TEXT("StatusItemBorder"),					Theme::BORDER_NONE,
		COLOR_STATUSITEMBORDER,						false},
	{TEXT("StatusBottomItemBorder"),			Theme::BORDER_NONE,
		COLOR_STATUSBOTTOMITEMBORDER,				false},
	{TEXT("StatusHighlightBorder"),				Theme::BORDER_NONE,
		COLOR_STATUSHIGHLIGHTBORDER,				false},
	{TEXT("TitleBarBorder"),					Theme::BORDER_RAISED,
		COLOR_TITLEBARBORDER,						true},
	{TEXT("TitleBarCaptionBorder"),				Theme::BORDER_NONE,
		COLOR_TITLEBARTEXTBORDER,					false},
	{TEXT("TitleBarIconBorder"),				Theme::BORDER_NONE,
		COLOR_TITLEBARICONBORDER,					false},
	{TEXT("TitleBarHighlightBorder"),			Theme::BORDER_NONE,
		COLOR_TITLEBARHIGHLIGHTBORDER,				false},
	{TEXT("SideBarBorder"),						Theme::BORDER_RAISED,
		COLOR_SIDEBARBORDER,						true},
	{TEXT("SideBarItemBorder"),					Theme::BORDER_NONE,
		COLOR_SIDEBARITEMBORDER,					false},
	{TEXT("SideBarHighlightBorder"),			Theme::BORDER_NONE,
		COLOR_SIDEBARHIGHLIGHTBORDER,				false},
	{TEXT("SideBarCheckBorder"),				Theme::BORDER_SUNKEN,
		COLOR_SIDEBARCHECKBORDER,					false},
	{TEXT("ProgramGuideStatusBorder"),			Theme::BORDER_SUNKEN,
		COLOR_STATUSBORDER,							true},
	{TEXT("PanelTabBorder"),					Theme::BORDER_SOLID,
		COLOR_PANELTABBORDER,						false},
	{TEXT("PanelCurTabBorder"),					Theme::BORDER_SOLID,
		COLOR_PANELCURTABBORDER,					false},
	{TEXT("PanelTabMarginBorder"),				Theme::BORDER_NONE,
		COLOR_PANELTABMARGINBORDER,					false},
	{TEXT("PanelTitleBorder"),					Theme::BORDER_RAISED,
		COLOR_PANELTITLEBORDER,						false},
	{TEXT("ProgramListPanelEventBorder"),		Theme::BORDER_NONE,
		COLOR_PROGRAMLISTPANEL_EVENTBORDER,			false},
	{TEXT("ProgramListPanelCurEventBorder"),	Theme::BORDER_NONE,
		COLOR_PROGRAMLISTPANEL_CUREVENTBORDER,		false},
	{TEXT("ProgramListPanelTitleBorder"),		Theme::BORDER_NONE,
		COLOR_PROGRAMLISTPANEL_TITLEBORDER,			false},
	{TEXT("ProgramListPanelCurTitleBorder"),	Theme::BORDER_NONE,
		COLOR_PROGRAMLISTPANEL_CURTITLEBORDER,		false},
	{TEXT("ChannelPanelChannelNameBorder"),		Theme::BORDER_NONE,
		COLOR_CHANNELPANEL_CHANNELNAMEBORDER,		false},
	{TEXT("ChannelPanelCurChannelNameBorder"),	Theme::BORDER_NONE,
		COLOR_CHANNELPANEL_CURCHANNELNAMEBORDER,	false},
	{TEXT("ChannelPanelEventNameBorder"),		Theme::BORDER_NONE,
		COLOR_CHANNELPANEL_EVENTNAME1BORDER,		false},
	{TEXT("ChannelPanelEventName2Border"),		Theme::BORDER_NONE,
		COLOR_CHANNELPANEL_EVENTNAME2BORDER,		false},
	{TEXT("ChannelPanelCurEventNameBorder"),	Theme::BORDER_NONE,
		COLOR_CHANNELPANEL_CUREVENTNAME1BORDER,		false},
	{TEXT("ChannelPanelCurEventName2Border"),	Theme::BORDER_NONE,
		COLOR_CHANNELPANEL_CUREVENTNAME2BORDER,		false},
	{TEXT("ControlPanelItemBorder"),			Theme::BORDER_NONE,
		COLOR_CONTROLPANELITEMBORDER,				false},
	{TEXT("ControlPanelHighlightBorder"),		Theme::BORDER_NONE,
		COLOR_CONTROLPANELHIGHLIGHTBORDER,			false},
};

const CColorScheme::StyleInfo CColorScheme::m_StyleList[NUM_STYLES] = {
	{GRADIENT_STATUSBACK,						BORDER_STATUSITEM,
		COLOR_STATUSTEXT},
	{GRADIENT_STATUSBOTTOMITEMBACK,				BORDER_STATUSBOTTOMITEM,
		COLOR_STATUSBOTTOMITEMTEXT},
	{GRADIENT_STATUSHIGHLIGHTBACK,				BORDER_STATUSHIGHLIGHT,
		COLOR_STATUSHIGHLIGHTTEXT},
	{GRADIENT_TITLEBARBACK,						BORDER_TITLEBARCAPTION,
		COLOR_TITLEBARTEXT},
	{GRADIENT_TITLEBARICON,						BORDER_TITLEBARICON,
		COLOR_TITLEBARICON},
	{GRADIENT_TITLEBARHIGHLIGHTBACK,			BORDER_TITLEBARHIGHLIGHT,
		COLOR_TITLEBARHIGHLIGHTICON},
	{GRADIENT_SIDEBARBACK,						BORDER_SIDEBARITEM,
		COLOR_SIDEBARICON},
	{GRADIENT_SIDEBARHIGHLIGHTBACK,				BORDER_SIDEBARHIGHLIGHT,
		COLOR_SIDEBARHIGHLIGHTICON},
	{GRADIENT_SIDEBARCHECKBACK,					BORDER_SIDEBARCHECK,
		COLOR_SIDEBARCHECKICON},
	{GRADIENT_PANELTABBACK,						BORDER_PANEL_TAB,
		COLOR_PANELTABTEXT},
	{GRADIENT_PANELCURTABBACK,					BORDER_PANEL_CURTAB,
		COLOR_PANELCURTABTEXT},
	{GRADIENT_PANELTABMARGIN,					BORDER_PANEL_TABMARGIN,
		-1},
	{GRADIENT_PANELTITLEBACK,					BORDER_PANEL_TITLE,
		COLOR_PANELTITLETEXT},
	{GRADIENT_PROGRAMLISTPANEL_EVENTBACK,		BORDER_PROGRAMLISTPANEL_EVENT,
		COLOR_PROGRAMLISTPANEL_EVENTTEXT},
	{GRADIENT_PROGRAMLISTPANEL_CUREVENTBACK,	BORDER_PROGRAMLISTPANEL_CUREVENT,
		COLOR_PROGRAMLISTPANEL_CUREVENTTEXT},
	{GRADIENT_PROGRAMLISTPANEL_TITLEBACK,		BORDER_PROGRAMLISTPANEL_TITLE,
		COLOR_PROGRAMLISTPANEL_TITLETEXT},
	{GRADIENT_PROGRAMLISTPANEL_CURTITLEBACK,	BORDER_PROGRAMLISTPANEL_CURTITLE,
		COLOR_PROGRAMLISTPANEL_CURTITLETEXT},
	{GRADIENT_CHANNELPANEL_CHANNELNAMEBACK,		BORDER_CHANNELPANEL_CHANNELNAME,
		COLOR_CHANNELPANEL_CHANNELNAMETEXT},
	{GRADIENT_CHANNELPANEL_CURCHANNELNAMEBACK,	BORDER_CHANNELPANEL_CURCHANNELNAME,
		COLOR_CHANNELPANEL_CURCHANNELNAMETEXT},
	{GRADIENT_CHANNELPANEL_EVENTNAMEBACK1,		BORDER_CHANNELPANEL_EVENTNAME1,
		COLOR_CHANNELPANEL_EVENTNAME1TEXT},
	{GRADIENT_CHANNELPANEL_EVENTNAMEBACK2,		BORDER_CHANNELPANEL_EVENTNAME2,
		COLOR_CHANNELPANEL_EVENTNAME2TEXT},
	{GRADIENT_CHANNELPANEL_CUREVENTNAMEBACK1,	BORDER_CHANNELPANEL_CUREVENTNAME1,
		COLOR_CHANNELPANEL_CUREVENTNAME1TEXT},
	{GRADIENT_CHANNELPANEL_CUREVENTNAMEBACK2,	BORDER_CHANNELPANEL_CUREVENTNAME2,
		COLOR_CHANNELPANEL_CUREVENTNAME2TEXT},
	{GRADIENT_CONTROLPANELBACK,					BORDER_CONTROLPANELITEM,
		COLOR_CONTROLPANELTEXT},
	{GRADIENT_CONTROLPANELHIGHLIGHTBACK,		BORDER_CONTROLPANELHIGHLIGHTITEM,
		COLOR_CONTROLPANELHIGHLIGHTTEXT},
};


CColorScheme::CColorScheme()
{
	SetDefault();
	::ZeroMemory(m_LoadedFlags,sizeof(m_LoadedFlags));
}


CColorScheme::CColorScheme(const CColorScheme &ColorScheme)
{
	*this=ColorScheme;
}


CColorScheme::~CColorScheme()
{
}


CColorScheme &CColorScheme::operator=(const CColorScheme &ColorScheme)
{
	if (&ColorScheme!=this) {
		::CopyMemory(m_ColorList,ColorScheme.m_ColorList,sizeof(m_ColorList));
		::CopyMemory(m_GradientList,ColorScheme.m_GradientList,sizeof(m_GradientList));
		::CopyMemory(m_BorderList,ColorScheme.m_BorderList,sizeof(m_BorderList));
		m_Name=ColorScheme.m_Name;
		m_FileName=ColorScheme.m_FileName;
		::CopyMemory(m_LoadedFlags,ColorScheme.m_LoadedFlags,sizeof(m_LoadedFlags));
	}
	return *this;
}


COLORREF CColorScheme::GetColor(int Type) const
{
	if (Type<0 || Type>=NUM_COLORS)
		return CLR_INVALID;
	return m_ColorList[Type];
}


COLORREF CColorScheme::GetColor(LPCTSTR pszText) const
{
	for (int i=0;i<NUM_COLORS;i++) {
		if (::lstrcmpi(m_ColorInfoList[i].pszText,pszText)==0)
			return m_ColorList[i];
	}
	return CLR_INVALID;
}


bool CColorScheme::SetColor(int Type,COLORREF Color)
{
	if (Type<0 || Type>=NUM_COLORS)
		return false;
	m_ColorList[Type]=Color;
	return true;
}


Theme::GradientType CColorScheme::GetGradientType(int Gradient) const
{
	if (Gradient<0 || Gradient>=NUM_GRADIENTS)
		return Theme::GRADIENT_NORMAL;
	return m_GradientList[Gradient].Type;
}


Theme::GradientType CColorScheme::GetGradientType(LPCTSTR pszText) const
{
	for (int i=0;i<NUM_GRADIENTS;i++) {
		if (::lstrcmpi(m_GradientInfoList[i].pszText,pszText)==0)
			return m_GradientList[i].Type;
	}
	return Theme::GRADIENT_NORMAL;
}


bool CColorScheme::SetGradientStyle(int Gradient,const GradientStyle &Style)
{
	if (Gradient<0 || Gradient>=NUM_GRADIENTS)
		return false;
	m_GradientList[Gradient].Type=Style.Type;
	m_GradientList[Gradient].Direction=Style.Direction;
	return true;
}


bool CColorScheme::GetGradientStyle(int Gradient,GradientStyle *pStyle) const
{
	if (Gradient<0 || Gradient>=NUM_GRADIENTS)
		return false;
	*pStyle=m_GradientList[Gradient];
	return true;
}


bool CColorScheme::GetGradientInfo(int Gradient,Theme::GradientInfo *pInfo) const
{
	if (Gradient<0 || Gradient>=NUM_GRADIENTS)
		return false;
	pInfo->Type=m_GradientList[Gradient].Type;
	pInfo->Direction=m_GradientList[Gradient].Direction;
	pInfo->Color1=m_ColorList[m_GradientInfoList[Gradient].Color1];
	pInfo->Color2=m_ColorList[m_GradientInfoList[Gradient].Color2];
	return true;
}


Theme::BorderType CColorScheme::GetBorderType(int Border) const
{
	if (Border<0 || Border>=NUM_BORDERS)
		return Theme::BORDER_NONE;
	return m_BorderList[Border];
}


bool CColorScheme::SetBorderType(int Border,Theme::BorderType Type)
{
	if (Border<0 || Border>=NUM_BORDERS
			|| Type<Theme::BORDER_NONE || Type>Theme::BORDER_RAISED)
		return false;
	m_BorderList[Border]=Type;
	return true;
}


bool CColorScheme::GetBorderInfo(int Border,Theme::BorderInfo *pInfo) const
{
	if (Border<0 || Border>=NUM_BORDERS)
		return false;
	pInfo->Type=m_BorderList[Border];
	pInfo->Color=m_ColorList[m_BorderInfoList[Border].Color];
	return true;
}


bool CColorScheme::GetStyle(int Type,Theme::Style *pStyle) const
{
	if (Type<0 || Type>=NUM_STYLES)
		return false;

	const StyleInfo &Info=m_StyleList[Type];
	GetGradientInfo(Info.Gradient,&pStyle->Gradient);
	if (Info.Border>=0)
		GetBorderInfo(Info.Border,&pStyle->Border);
	else
		pStyle->Border.Type=Theme::BORDER_NONE;
	if (Info.TextColor>=0)
		pStyle->TextColor=m_ColorList[Info.TextColor];
	return true;
}


bool CColorScheme::SetName(LPCTSTR pszName)
{
	return m_Name.Set(pszName);
}


bool CColorScheme::Load(LPCTSTR pszFileName)
{
	CSettings Settings;
	TCHAR szText[128];
	int i;

	if (!Settings.Open(pszFileName,TEXT("ColorScheme"),CSettings::OPEN_READ))
		return false;
	if (Settings.Read(TEXT("Name"),szText,lengthof(szText)))
		SetName(szText);
	::ZeroMemory(m_LoadedFlags,sizeof(m_LoadedFlags));
	for (i=0;i<NUM_COLORS;i++) {
		if (Settings.ReadColor(m_ColorInfoList[i].pszText,&m_ColorList[i]))
			SetLoadedFlag(i);
	}
	for (i=0;i<NUM_COLORS;i++) {
		if (IsLoaded(i)) {
			for (int j=0;j<NUM_GRADIENTS;j++) {
				if (m_GradientInfoList[j].Color1==i
						|| m_GradientInfoList[j].Color2==i) {
					if (m_GradientInfoList[j].Color1==i
							&& !IsLoaded(m_GradientInfoList[j].Color2)) {
						m_ColorList[m_GradientInfoList[j].Color2]=m_ColorList[i];
						SetLoadedFlag(m_GradientInfoList[j].Color2);
					}
					m_GradientList[j].Type=Theme::GRADIENT_NORMAL;
					break;
				}
			}
		} else if (i==COLOR_SPLITTER) {
			m_ColorList[i]=::GetSysColor(COLOR_3DFACE);
			SetLoadedFlag(i);
		} else {
			static const struct {
				int To,From;
			} Map[] = {
				{COLOR_STATUSBORDER,						COLOR_STATUSBACK1},
				{COLOR_STATUSBOTTOMITEMBACK1,				COLOR_STATUSBACK2},
				{COLOR_STATUSBOTTOMITEMBACK2,				COLOR_STATUSBOTTOMITEMBACK1},
				{COLOR_STATUSBOTTOMITEMTEXT,				COLOR_STATUSTEXT},
				{COLOR_STATUSBOTTOMITEMBORDER,				COLOR_STATUSBOTTOMITEMBACK1},
				{COLOR_PROGRAMLISTPANEL_CUREVENTBACK1,		COLOR_PROGRAMLISTPANEL_EVENTBACK1},
				{COLOR_PROGRAMLISTPANEL_CUREVENTBACK2,		COLOR_PROGRAMLISTPANEL_EVENTBACK2},
				{COLOR_PROGRAMLISTPANEL_CUREVENTTEXT,		COLOR_PROGRAMLISTPANEL_EVENTTEXT},
				{COLOR_PROGRAMLISTPANEL_CURTITLEBACK1,		COLOR_PROGRAMLISTPANEL_TITLEBACK1},
				{COLOR_PROGRAMLISTPANEL_CURTITLEBACK2,		COLOR_PROGRAMLISTPANEL_TITLEBACK2},
				{COLOR_PROGRAMLISTPANEL_CURTITLETEXT,		COLOR_PROGRAMLISTPANEL_TITLETEXT},
				{COLOR_PANELTABLINE,						COLOR_PANELTABBORDER},
				{COLOR_PANELTITLEBORDER,					COLOR_PANELTITLEBACK1},
				{COLOR_CHANNELPANEL_CURCHANNELNAMEBACK1,	COLOR_CHANNELPANEL_CHANNELNAMEBACK1},
				{COLOR_CHANNELPANEL_CURCHANNELNAMEBACK2,	COLOR_CHANNELPANEL_CHANNELNAMEBACK2},
				{COLOR_CHANNELPANEL_CURCHANNELNAMETEXT,		COLOR_CHANNELPANEL_CHANNELNAMETEXT},
				{COLOR_CHANNELPANEL_EVENTNAME2BACK1,		COLOR_CHANNELPANEL_EVENTNAME1BACK1},
				{COLOR_CHANNELPANEL_EVENTNAME2BACK2,		COLOR_CHANNELPANEL_EVENTNAME1BACK2},
				{COLOR_CHANNELPANEL_EVENTNAME2TEXT,			COLOR_CHANNELPANEL_EVENTNAME1TEXT},
				{COLOR_CHANNELPANEL_EVENTNAME2BORDER,		COLOR_CHANNELPANEL_EVENTNAME1BORDER},
				{COLOR_CHANNELPANEL_CUREVENTNAME1BACK1,		COLOR_CHANNELPANEL_EVENTNAME1BACK1},
				{COLOR_CHANNELPANEL_CUREVENTNAME1BACK2,		COLOR_CHANNELPANEL_EVENTNAME1BACK2},
				{COLOR_CHANNELPANEL_CUREVENTNAME1TEXT,		COLOR_CHANNELPANEL_EVENTNAME1TEXT},
				{COLOR_CHANNELPANEL_CUREVENTNAME1BORDER,	COLOR_CHANNELPANEL_EVENTNAME1BORDER},
				{COLOR_CHANNELPANEL_CUREVENTNAME2BACK1,		COLOR_CHANNELPANEL_CUREVENTNAME1BACK1},
				{COLOR_CHANNELPANEL_CUREVENTNAME2BACK2,		COLOR_CHANNELPANEL_CUREVENTNAME1BACK2},
				{COLOR_CHANNELPANEL_CUREVENTNAME2TEXT,		COLOR_CHANNELPANEL_CUREVENTNAME1TEXT},
				{COLOR_CHANNELPANEL_CUREVENTNAME2BORDER,	COLOR_CHANNELPANEL_CUREVENTNAME1BORDER},
				{COLOR_CONTROLPANELBACK1,					COLOR_PANELBACK},
				{COLOR_CONTROLPANELBACK2,					COLOR_PANELBACK},
				{COLOR_CONTROLPANELTEXT,					COLOR_PANELTEXT},
				{COLOR_CONTROLPANELMARGIN,					COLOR_PANELBACK},
				{COLOR_CAPTIONPANELBACK,					COLOR_PROGRAMINFOBACK},
				{COLOR_CAPTIONPANELTEXT,					COLOR_PROGRAMINFOTEXT},
				{COLOR_TITLEBARBACK1,						COLOR_STATUSBACK1},
				{COLOR_TITLEBARBACK2,						COLOR_STATUSBACK2},
				{COLOR_TITLEBARTEXT,						COLOR_STATUSTEXT},
				{COLOR_TITLEBARICONBACK1,					COLOR_TITLEBARBACK1},
				{COLOR_TITLEBARICONBACK2,					COLOR_TITLEBARBACK2},
				{COLOR_TITLEBARICON,						COLOR_TITLEBARTEXT},
				{COLOR_TITLEBARHIGHLIGHTBACK1,				COLOR_STATUSHIGHLIGHTBACK1},
				{COLOR_TITLEBARHIGHLIGHTBACK2,				COLOR_STATUSHIGHLIGHTBACK2},
				{COLOR_TITLEBARHIGHLIGHTICON,				COLOR_STATUSHIGHLIGHTTEXT},
				{COLOR_TITLEBARBORDER,						COLOR_TITLEBARBACK1},
				{COLOR_SIDEBARBACK1,						COLOR_STATUSBACK1},
				{COLOR_SIDEBARBACK2,						COLOR_STATUSBACK2},
				{COLOR_SIDEBARICON,							COLOR_STATUSTEXT},
				{COLOR_SIDEBARHIGHLIGHTBACK1,				COLOR_STATUSHIGHLIGHTBACK1},
				{COLOR_SIDEBARHIGHLIGHTBACK2,				COLOR_STATUSHIGHLIGHTBACK2},
				{COLOR_SIDEBARHIGHLIGHTICON,				COLOR_STATUSHIGHLIGHTTEXT},
				{COLOR_SIDEBARCHECKBACK1,					COLOR_SIDEBARBACK1},
				{COLOR_SIDEBARCHECKBACK2,					COLOR_SIDEBARBACK2},
				{COLOR_SIDEBARCHECKICON,					COLOR_SIDEBARICON},
				{COLOR_SIDEBARCHECKBORDER,					COLOR_SIDEBARCHECKBACK2},
				{COLOR_SIDEBARBORDER,						COLOR_SIDEBARBACK1},
				{COLOR_PROGRAMGUIDECURCHANNELBACK1,			COLOR_PROGRAMGUIDECHANNELBACK1},
				{COLOR_PROGRAMGUIDECURCHANNELBACK2,			COLOR_PROGRAMGUIDECHANNELBACK2},
				{COLOR_PROGRAMGUIDECURCHANNELTEXT,			COLOR_PROGRAMGUIDECHANNELTEXT},
				{COLOR_PROGRAMGUIDETIMELINE,				COLOR_PROGRAMGUIDETIMETEXT},
			};

			for (int j=0;j<lengthof(Map);j++) {
				if (Map[j].To==i && IsLoaded(Map[j].From)) {
					m_ColorList[i]=m_ColorList[Map[j].From];
					SetLoadedFlag(i);
					break;
				}
			}
		}
	}

	for (i=0;i<NUM_GRADIENTS;i++) {
		if (Settings.Read(m_GradientInfoList[i].pszText,szText,lengthof(szText))) {
			if (szText[0]=='\0' || ::lstrcmpi(szText,TEXT("normal"))==0)
				m_GradientList[i].Type=Theme::GRADIENT_NORMAL;
			else if (::lstrcmpi(szText,TEXT("glossy"))==0)
				m_GradientList[i].Type=Theme::GRADIENT_GLOSSY;
			else if (::lstrcmpi(szText,TEXT("interlaced"))==0)
				m_GradientList[i].Type=Theme::GRADIENT_INTERLACED;
		} else {
			switch (i) {
			case GRADIENT_TITLEBARICON:
				m_GradientList[i].Type=m_GradientList[GRADIENT_TITLEBARBACK].Type;
				break;
			case GRADIENT_SIDEBARCHECKBACK:
				m_GradientList[i].Type=m_GradientList[GRADIENT_SIDEBARBACK].Type;
				break;
			}
		}

		TCHAR szName[128];
		::wsprintf(szName,TEXT("%sDirection"),m_GradientInfoList[i].pszText);
		m_GradientList[i].Direction=m_GradientInfoList[i].Direction;
		if (Settings.Read(szName,szText,lengthof(szText))) {
			for (int j=0;j<lengthof(GradientDirectionList);j++) {
				if (::lstrcmpi(szText,GradientDirectionList[j])==0) {
					m_GradientList[i].Direction=(Theme::GradientDirection)j;
					break;
				}
			}
		} else {
			switch (i) {
			case GRADIENT_TITLEBARICON:
				m_GradientList[i].Direction=m_GradientList[GRADIENT_TITLEBARBACK].Direction;
				break;
			case GRADIENT_SIDEBARCHECKBACK:
				m_GradientList[i].Direction=m_GradientList[GRADIENT_SIDEBARBACK].Direction;
				break;
			}
		}
	}

	Settings.Close();

	for (i=0;i<NUM_BORDERS;i++)
		m_BorderList[i]=m_BorderInfoList[i].DefaultType;
	if (Settings.Open(pszFileName,TEXT("Style"),CSettings::OPEN_READ)) {
		for (i=0;i<NUM_BORDERS;i++) {
			if (Settings.Read(m_BorderInfoList[i].pszText,szText,lengthof(szText))) {
				if (::lstrcmpi(szText,TEXT("none"))==0) {
					if (!m_BorderInfoList[i].fAlways)
						m_BorderList[i]=Theme::BORDER_NONE;
				} else if (::lstrcmpi(szText,TEXT("solid"))==0)
					m_BorderList[i]=Theme::BORDER_SOLID;
				else if (::lstrcmpi(szText,TEXT("sunken"))==0)
					m_BorderList[i]=Theme::BORDER_SUNKEN;
				else if (::lstrcmpi(szText,TEXT("raised"))==0)
					m_BorderList[i]=Theme::BORDER_RAISED;
			}
		}
		Settings.Close();
	}

	SetFileName(pszFileName);
	return true;
}


bool CColorScheme::Save(LPCTSTR pszFileName) const
{
	CSettings Settings;
	int i;

	if (!Settings.Open(pszFileName,TEXT("ColorScheme"),CSettings::OPEN_WRITE))
		return false;
	Settings.Write(TEXT("Name"),m_Name.GetSafe());
	for (i=0;i<NUM_COLORS;i++)
		Settings.WriteColor(m_ColorInfoList[i].pszText,m_ColorList[i]);
	for (i=0;i<NUM_GRADIENTS;i++) {
		static const LPCTSTR pszTypeName[] = {
			TEXT("normal"),	TEXT("glossy"), TEXT("interlaced")
		};
		TCHAR szName[128];

		Settings.Write(m_GradientInfoList[i].pszText,pszTypeName[m_GradientList[i].Type]);
		::wsprintf(szName,TEXT("%sDirection"),m_GradientInfoList[i].pszText);
		Settings.Write(szName,GradientDirectionList[m_GradientList[i].Direction]);
	}
	Settings.Close();

	if (Settings.Open(pszFileName,TEXT("Style"),CSettings::OPEN_WRITE)) {
		static const LPCTSTR pszTypeName[] = {
			TEXT("none"),	TEXT("solid"),	TEXT("sunken"),	TEXT("raised")
		};

		for (i=0;i<NUM_BORDERS;i++)
			Settings.Write(m_BorderInfoList[i].pszText,pszTypeName[m_BorderList[i]]);
		Settings.Close();
	}
	return true;
}


bool CColorScheme::SetFileName(LPCTSTR pszFileName)
{
	return m_FileName.Set(pszFileName);
}


void CColorScheme::SetDefault()
{
	int i;

	for (i=0;i<NUM_COLORS;i++)
		m_ColorList[i]=m_ColorInfoList[i].DefaultColor;
	for (i=0;i<NUM_GRADIENTS;i++) {
		m_GradientList[i].Type=Theme::GRADIENT_NORMAL;
		m_GradientList[i].Direction=m_GradientInfoList[i].Direction;
	}
	for (i=0;i<NUM_BORDERS;i++)
		m_BorderList[i]=m_BorderInfoList[i].DefaultType;
}


LPCTSTR CColorScheme::GetColorName(int Type)
{
	if (Type<0 || Type>=NUM_COLORS)
		return NULL;
	return m_ColorInfoList[Type].pszName;
}


COLORREF CColorScheme::GetDefaultColor(int Type)
{
	if (Type<0 || Type>=NUM_COLORS)
		return CLR_INVALID;
	return m_ColorInfoList[Type].DefaultColor;
}


Theme::GradientType CColorScheme::GetDefaultGradientType(int Gradient)
{
	return Theme::GRADIENT_NORMAL;
}


bool CColorScheme::GetDefaultGradientStyle(int Gradient,GradientStyle *pStyle)
{
	if (Gradient<0 || Gradient>=NUM_GRADIENTS)
		return false;
	pStyle->Type=Theme::GRADIENT_NORMAL;
	pStyle->Direction=m_GradientInfoList[Gradient].Direction;
	return true;
}


bool CColorScheme::IsGradientDirectionEnabled(int Gradient)
{
	if (Gradient<0 || Gradient>=NUM_GRADIENTS)
		return false;
	return m_GradientInfoList[Gradient].fEnableDirection;
}


Theme::BorderType CColorScheme::GetDefaultBorderType(int Border)
{
	if (Border<0 || Border>=NUM_BORDERS)
		return Theme::BORDER_NONE;
	return m_BorderInfoList[Border].DefaultType;
}


bool CColorScheme::IsLoaded(int Type) const
{
	if (Type<0 || Type>=NUM_COLORS)
		return false;
	return (m_LoadedFlags[Type/32]&(1<<(Type%32)))!=0;
}


void CColorScheme::SetLoaded()
{
	::FillMemory(m_LoadedFlags,sizeof(m_LoadedFlags),0xFF);
}


int CColorScheme::GetColorGradient(int Type)
{
	for (int i=0;i<NUM_GRADIENTS;i++) {
		if (m_GradientInfoList[i].Color1==Type
				|| m_GradientInfoList[i].Color2==Type)
			return i;
	}
	return -1;
}


int CColorScheme::GetColorBorder(int Type)
{
	for (int i=0;i<NUM_BORDERS;i++) {
		if (m_BorderInfoList[i].Color==Type)
			return i;
	}
	return -1;
}


bool CColorScheme::IsBorderAlways(int Border)
{
	if (Border<0 || Border>=NUM_BORDERS)
		return false;
	return m_BorderInfoList[Border].fAlways;
}


void CColorScheme::SetLoadedFlag(int Color)
{
	m_LoadedFlags[Color/32]|=1<<(Color%32);
}




CColorSchemeList::CColorSchemeList()
	: m_NumColorSchemes(0)
	, m_ppList(NULL)
{
}


CColorSchemeList::~CColorSchemeList()
{
	Clear();
}


bool CColorSchemeList::Add(CColorScheme *pColorScheme)
{
	m_ppList=(CColorScheme**)realloc(m_ppList,(m_NumColorSchemes+1)*sizeof(CColorScheme*));
	m_ppList[m_NumColorSchemes++]=pColorScheme;
	return true;
}


bool CColorSchemeList::Load(LPCTSTR pszDirectory)
{
	HANDLE hFind;
	WIN32_FIND_DATA wfd;
	TCHAR szFileName[MAX_PATH];

	PathCombine(szFileName,pszDirectory,TEXT("*.httheme"));
	hFind=FindFirstFile(szFileName,&wfd);
	if (hFind!=INVALID_HANDLE_VALUE) {
		do {
			CColorScheme *pColorScheme;

			PathCombine(szFileName,pszDirectory,wfd.cFileName);
			pColorScheme=new CColorScheme;
			if (pColorScheme->Load(szFileName))
				Add(pColorScheme);
			else
				delete pColorScheme;
		} while (FindNextFile(hFind,&wfd));
		FindClose(hFind);
	}
	return true;
}


void CColorSchemeList::Clear()
{
	if (m_ppList!=NULL) {
		int i;

		for (i=0;i<m_NumColorSchemes;i++)
			delete m_ppList[i];
		free(m_ppList);
		m_ppList=NULL;
		m_NumColorSchemes=0;
	}
}


CColorScheme *CColorSchemeList::GetColorScheme(int Index)
{
	if (Index<0 || Index>=m_NumColorSchemes)
		return NULL;
	return m_ppList[Index];
}


bool CColorSchemeList::SetColorScheme(int Index,const CColorScheme *pColorScheme)
{
	if (Index<0 || Index>=m_NumColorSchemes)
		return true;
	*m_ppList[Index]=*pColorScheme;
	return true;
}




const LPCTSTR CColorSchemeOptions::m_pszExtension=TEXT(".httheme");


CColorSchemeOptions::CColorSchemeOptions()
	: m_pColorScheme(new CColorScheme)
	, m_pPreviewColorScheme(NULL)
	, m_pApplyFunc(NULL)
{
}


CColorSchemeOptions::~CColorSchemeOptions()
{
	delete m_pColorScheme;
	delete m_pPreviewColorScheme;
}


bool CColorSchemeOptions::Load(LPCTSTR pszFileName)
{
	return m_pColorScheme->Load(pszFileName);
}


bool CColorSchemeOptions::Save(LPCTSTR pszFileName) const
{
	return m_pColorScheme->Save(pszFileName);
}


bool CColorSchemeOptions::SetApplyCallback(ApplyFunc pCallback)
{
	m_pApplyFunc=pCallback;
	return true;
}


bool CColorSchemeOptions::ApplyColorScheme() const
{
	return Apply(m_pColorScheme);
}


bool CColorSchemeOptions::Apply(const CColorScheme *pColorScheme) const
{
	if (m_pApplyFunc==NULL)
		return false;
	return m_pApplyFunc(pColorScheme);
}


COLORREF CColorSchemeOptions::GetColor(int Type) const
{
	if (m_pPreviewColorScheme!=NULL)
		return m_pPreviewColorScheme->GetColor(Type);
	return m_pColorScheme->GetColor(Type);
}


COLORREF CColorSchemeOptions::GetColor(LPCTSTR pszText) const
{
	if (m_pPreviewColorScheme!=NULL)
		return m_pPreviewColorScheme->GetColor(pszText);
	return m_pColorScheme->GetColor(pszText);
}


void CColorSchemeOptions::GetCurrentSettings(CColorScheme *pColorScheme)
{
	int i;

	for (i=0;i<CColorScheme::NUM_COLORS;i++)
		pColorScheme->SetColor(i,
							   (COLORREF)DlgListBox_GetItemData(m_hDlg,IDC_COLORSCHEME_LIST,i));
	for (int i=0;i<CColorScheme::NUM_GRADIENTS;i++)
		pColorScheme->SetGradientStyle(i,m_GradientList[i]);
	for (int i=0;i<CColorScheme::NUM_BORDERS;i++)
		pColorScheme->SetBorderType(i,m_BorderList[i]);
}


bool CColorSchemeOptions::GetThemesDirectory(LPTSTR pszDirectory,int MaxLength,bool fCreate)
{
	GetAppClass().GetAppDirectory(pszDirectory);
	::PathAppend(pszDirectory,TEXT("Themes"));
	if (fCreate && !::PathIsDirectory(pszDirectory))
		::CreateDirectory(pszDirectory,NULL);
	return true;
}


CColorSchemeOptions *CColorSchemeOptions::GetThis(HWND hDlg)
{
	return static_cast<CColorSchemeOptions*>(GetOptions(hDlg));
}


INT_PTR CALLBACK CColorSchemeOptions::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg) {
	case WM_INITDIALOG:
		{
			CColorSchemeOptions *pThis=static_cast<CColorSchemeOptions*>(OnInitDialog(hDlg,lParam));
			TCHAR szDirectory[MAX_PATH];
			int i;
			int Index;

			GetThemesDirectory(szDirectory,lengthof(szDirectory));
			pThis->m_PresetList.Load(szDirectory);
			for (i=0;i<pThis->m_PresetList.NumColorSchemes();i++) {
				LPCTSTR pszName=pThis->m_PresetList.GetColorScheme(i)->GetName();

				Index=(int)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_ADDSTRING,0,
									(LPARAM)(pszName!=NULL?pszName:TEXT("")));
				SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_SETITEMDATA,Index,i);
			}
			SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_SETCUEBANNER,0,
							   reinterpret_cast<LPARAM>(L"<zFIĂ>"));

			for (i=0;i<CColorScheme::NUM_COLORS;i++) {
				SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_ADDSTRING,0,pThis->m_pColorScheme->GetColor(i));
			}
			HDC hdc=GetDC(GetDlgItem(hDlg,IDC_COLORSCHEME_LIST));
			HFONT hfontOld=SelectFont(hdc,(HFONT)SendDlgItemMessage(hDlg,
										IDC_COLORSCHEME_LIST,WM_GETFONT,0,0));
			long MaxWidth=0;
			for (i=0;i<CColorScheme::NUM_COLORS;i++) {
				LPCTSTR pszName=CColorScheme::GetColorName(i);
				SIZE sz;
				GetTextExtentPoint32(hdc,pszName,lstrlen(pszName),&sz);
				if (sz.cx>MaxWidth)
					MaxWidth=sz.cx;
			}
			SelectFont(hdc,hfontOld);
			ReleaseDC(GetDlgItem(hDlg,IDC_COLORSCHEME_LIST),hdc);
			SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_SETHORIZONTALEXTENT,
				SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETITEMHEIGHT,0,0)*2+MaxWidth+2,0);
			ExtendListBox(GetDlgItem(hDlg,IDC_COLORSCHEME_LIST));

			for (int i=0;i<CColorScheme::NUM_GRADIENTS;i++)
				pThis->m_pColorScheme->GetGradientStyle(i,&pThis->m_GradientList[i]);
			for (int i=0;i<CColorScheme::NUM_BORDERS;i++)
				pThis->m_BorderList[i]=pThis->m_pColorScheme->GetBorderType(i);

			RECT rc;
			static const RGBQUAD BaseColors[18] = {
				{0x00, 0x00, 0xFF},
				{0x00, 0x66, 0xFF},
				{0x00, 0xCC, 0xFF},
				{0x00, 0xFF, 0xFF},
				{0x00, 0xFF, 0xCC},
				{0x00, 0xFF, 0x66},
				{0x00, 0xFF, 0x00},
				{0x66, 0xFF, 0x00},
				{0xCC, 0xFF, 0x00},
				{0xFF, 0xFF, 0x00},
				{0xFF, 0xCC, 0x00},
				{0xFF, 0x66, 0x00},
				{0xFF, 0x00, 0x00},
				{0xFF, 0x00, 0x66},
				{0xFF, 0x00, 0xCC},
				{0xFF, 0x00, 0xFF},
				{0xCC, 0x00, 0xFF},
				{0x66, 0x00, 0xFF},
			};
			RGBQUAD Palette[256];
			int j,k;

			CColorPalette::Initialize(GetWindowInstance(hDlg));
			pThis->m_ColorPalette.Create(hDlg,WS_CHILD | WS_VISIBLE,0,IDC_COLORSCHEME_PALETTE);
			GetWindowRect(GetDlgItem(hDlg,IDC_COLORSCHEME_PALETTEPLACE),&rc);
			MapWindowPoints(NULL,hDlg,(LPPOINT)&rc,2);
			pThis->m_ColorPalette.SetPosition(&rc);
			for (i=0;i<lengthof(BaseColors);i++) {
				RGBQUAD Color=BaseColors[i%2*(lengthof(BaseColors)/2)+i/2];

				for (j=0;j<4;j++) {
					Palette[i*8+j].rgbBlue=(Color.rgbBlue*(j+1))/4;
					Palette[i*8+j].rgbGreen=(Color.rgbGreen*(j+1))/4;
					Palette[i*8+j].rgbRed=(Color.rgbRed*(j+1))/4;
				}
				for (;j<8;j++) {
					Palette[i*8+j].rgbBlue=Color.rgbBlue+(255-Color.rgbBlue)*(j-3)/5;
					Palette[i*8+j].rgbGreen=Color.rgbGreen+(255-Color.rgbGreen)*(j-3)/5;
					Palette[i*8+j].rgbRed=Color.rgbRed+(255-Color.rgbRed)*(j-3)/5;
				}
			}
			i=lengthof(BaseColors)*8;
			for (j=0;j<16;j++) {
				Palette[i].rgbBlue=(255*j)/15;
				Palette[i].rgbGreen=(255*j)/15;
				Palette[i].rgbRed=(255*j)/15;
				i++;
			}
			for (j=0;j<CColorScheme::NUM_COLORS;j++) {
				COLORREF cr=pThis->m_pColorScheme->GetColor(j);

				for (k=0;k<i;k++) {
					if (cr==RGB(Palette[k].rgbRed,Palette[k].rgbGreen,Palette[k].rgbBlue))
						break;
				}
				if (k==i) {
					Palette[i].rgbBlue=GetBValue(cr);
					Palette[i].rgbGreen=GetGValue(cr);
					Palette[i].rgbRed=GetRValue(cr);
					i++;
				}
			}
			if (i<lengthof(Palette))
				ZeroMemory(&Palette[i],(lengthof(Palette)-i)*sizeof(RGBQUAD));
			pThis->m_ColorPalette.SetPalette(Palette,lengthof(Palette));
		}
		return TRUE;

	case WM_MEASUREITEM:
		{
			LPMEASUREITEMSTRUCT pmis=reinterpret_cast<LPMEASUREITEMSTRUCT>(lParam);

			pmis->itemHeight=7*HIWORD(GetDialogBaseUnits())/8;
			if (pmis->CtlID==IDC_COLORSCHEME_PRESET)
				pmis->itemHeight+=4;
		}
		return TRUE;

	case WM_DRAWITEM:
		{
			CColorSchemeOptions *pThis=GetThis(hDlg);
			LPDRAWITEMSTRUCT pdis=reinterpret_cast<LPDRAWITEMSTRUCT>(lParam);

			if (pdis->CtlID==IDC_COLORSCHEME_PRESET) {
				switch (pdis->itemAction) {
				case ODA_DRAWENTIRE:
				case ODA_SELECT:
					if ((int)pdis->itemID<0) {
						::FillRect(pdis->hDC,&pdis->rcItem,
									reinterpret_cast<HBRUSH>(COLOR_WINDOW+1));
					} else {
						CColorScheme *pColorScheme=pThis->m_PresetList.GetColorScheme((int)pdis->itemData);
						Theme::Style Style;

						pColorScheme->GetStyle(
							(pdis->itemState&ODS_SELECTED)==0?
								CColorScheme::STYLE_STATUSITEM:
								CColorScheme::STYLE_STATUSHIGHLIGHTITEM,
							&Style);
						Theme::DrawStyleBackground(pdis->hDC,&pdis->rcItem,&Style);
						if (pColorScheme->GetName()!=NULL) {
							int OldBkMode;
							COLORREF crOldTextColor;
							RECT rc;
							HFONT hfont,hfontOld;

							if ((pdis->itemState&ODS_SELECTED)!=0) {
								LOGFONT lf;

								hfontOld=static_cast<HFONT>(::GetCurrentObject(pdis->hDC,OBJ_FONT));
								::GetObject(hfontOld,sizeof(LOGFONT),&lf);
								lf.lfWeight=FW_BOLD;
								hfont=::CreateFontIndirect(&lf);
								SelectFont(pdis->hDC,hfont);
							} else
								hfont=NULL;
							OldBkMode=::SetBkMode(pdis->hDC,TRANSPARENT);
							crOldTextColor=::SetTextColor(pdis->hDC,Style.TextColor);
							rc=pdis->rcItem;
							rc.left+=4;
							::DrawText(pdis->hDC,pColorScheme->GetName(),-1,&rc,
								DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX | DT_END_ELLIPSIS);
							::SetTextColor(pdis->hDC,crOldTextColor);
							::SetBkMode(pdis->hDC,OldBkMode);
							if (hfont!=NULL) {
								::SelectObject(pdis->hDC,hfontOld);
								::DeleteObject(hfont);
							}
						}
					}
					if ((pdis->itemState&ODS_FOCUS)==0)
						break;
				case ODA_FOCUS:
					::DrawFocusRect(pdis->hDC,&pdis->rcItem);
					break;
				}
			} else if (pdis->CtlID==IDC_COLORSCHEME_LIST) {
				switch (pdis->itemAction) {
				case ODA_DRAWENTIRE:
				case ODA_SELECT:
					{
						int BackSysColor;
						COLORREF BackColor,TextColor,OldTextColor;
						HBRUSH hbr,hbrOld;
						HPEN hpenOld;
						RECT rc;
						int OldBkMode;

						if ((pdis->itemState&ODS_SELECTED)==0) {
							BackSysColor=COLOR_WINDOW;
							TextColor=::GetSysColor(COLOR_WINDOWTEXT);
						} else {
							BackSysColor=COLOR_HIGHLIGHT;
							TextColor=::GetSysColor(COLOR_HIGHLIGHTTEXT);
						}
						BackColor=::GetSysColor(BackSysColor);
						int Border=CColorScheme::GetColorBorder((int)pdis->itemID);
						if (Border>=0 && pThis->m_BorderList[Border]==Theme::BORDER_NONE)
							TextColor=MixColor(TextColor,BackColor);
						::FillRect(pdis->hDC,&pdis->rcItem,reinterpret_cast<HBRUSH>(BackSysColor+1));
						hbr=::CreateSolidBrush((COLORREF)pdis->itemData);
						hbrOld=SelectBrush(pdis->hDC,hbr);
						hpenOld=SelectPen(pdis->hDC,::GetStockObject(BLACK_PEN));
						rc.left=pdis->rcItem.left+2;
						rc.top=pdis->rcItem.top+2;
						rc.bottom=pdis->rcItem.bottom-2;
						rc.right=rc.left+(rc.bottom-rc.top)*2;
						::Rectangle(pdis->hDC,rc.left,rc.top,rc.right,rc.bottom);
						::SelectObject(pdis->hDC,hpenOld);
						::SelectObject(pdis->hDC,hbrOld);
						::DeleteObject(hbr);
						OldBkMode=::SetBkMode(pdis->hDC,TRANSPARENT);
						OldTextColor=::SetTextColor(pdis->hDC,TextColor);
						rc.left=rc.right+2;
						rc.top=pdis->rcItem.top;
						rc.right=pdis->rcItem.right;
						rc.bottom=pdis->rcItem.bottom;
						::DrawText(pdis->hDC,CColorScheme::GetColorName(pdis->itemID),-1,&rc,
								   DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
						::SetTextColor(pdis->hDC,OldTextColor);
						::SetBkMode(pdis->hDC,OldBkMode);
					}
					if ((pdis->itemState&ODS_FOCUS)==0)
						break;
				case ODA_FOCUS:
					::DrawFocusRect(pdis->hDC,&pdis->rcItem);
					break;
				}
			}
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_COLORSCHEME_PRESET:
			if (HIWORD(wParam)==CBN_SELCHANGE) {
				int Sel=(int)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_GETCURSEL,0,0);

				if (Sel>=0) {
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int Index=(int)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_GETITEMDATA,Sel,0);
					const CColorScheme *pColorScheme=pThis->m_PresetList.GetColorScheme(Index);

					if (pColorScheme!=NULL) {
						int i;

						for (i=0;i<CColorScheme::NUM_COLORS;i++) {
							if (pColorScheme->IsLoaded(i))
								SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,
									LB_SETITEMDATA,i,pColorScheme->GetColor(i));
						}
						SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_SETSEL,FALSE,-1);
						InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);

						for (i=0;i<CColorScheme::NUM_GRADIENTS;i++)
							pColorScheme->GetGradientStyle(i,&pThis->m_GradientList[i]);
						for (i=0;i<CColorScheme::NUM_BORDERS;i++)
							pThis->m_BorderList[i]=pColorScheme->GetBorderType(i);

						pThis->m_ColorPalette.SetSel(-1);
						::SendMessage(hDlg,WM_COMMAND,IDC_COLORSCHEME_PREVIEW,0);
					}
				}
			}
			return TRUE;

		case IDC_COLORSCHEME_SAVE:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);
				TCHAR szName[MAX_PATH],szFileName[MAX_PATH];
				int Index;
				CColorScheme *pColorScheme=NULL;
				LPCTSTR pszFileName;

				GetDlgItemText(hDlg,IDC_COLORSCHEME_PRESET,szName,lengthof(szName));
				if (szName[0]=='\0') {
					MessageBox(hDlg,TEXT("O͂ĂB"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				pszFileName=szName;
				Index=(int)DlgComboBox_FindStringExact(hDlg,IDC_COLORSCHEME_PRESET,-1,szName);
				if (Index>=0) {
					pColorScheme=pThis->m_PresetList.GetColorScheme(
						(int)DlgComboBox_GetItemData(hDlg,IDC_COLORSCHEME_PRESET,Index));
					if (pColorScheme==NULL)
						break;
					pszFileName=PathFindFileName(pColorScheme->GetFileName());
				}
				GetThemesDirectory(szFileName,lengthof(szFileName),true);
				if (lstrlen(szFileName)+1+lstrlen(pszFileName)+lstrlen(m_pszExtension)>=MAX_PATH) {
					MessageBox(hDlg,TEXT("O܂B"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				PathAppend(szFileName,pszFileName);
				if (pszFileName==szName)
					lstrcat(szFileName,m_pszExtension);
				if (pColorScheme==NULL) {
					pColorScheme=new CColorScheme;
					pColorScheme->SetName(szName);
				}
				pThis->GetCurrentSettings(pColorScheme);
				if (!pColorScheme->Save(szFileName)) {
					if (Index<0)
						delete pColorScheme;
					MessageBox(hDlg,TEXT("ۑł܂B"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				if (Index<0) {
					pColorScheme->SetFileName(szFileName);
					pColorScheme->SetLoaded();
					pThis->m_PresetList.Add(pColorScheme);
					Index=(int)DlgComboBox_AddString(hDlg,IDC_COLORSCHEME_PRESET,szName);
					DlgComboBox_SetItemData(hDlg,IDC_COLORSCHEME_PRESET,Index,pThis->m_PresetList.NumColorSchemes()-1);
				}
				MessageBox(hDlg,TEXT("zFۑ܂B"),TEXT("ۑ"),MB_OK | MB_ICONINFORMATION);
			}
			return TRUE;

		case IDC_COLORSCHEME_DELETE:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);
				TCHAR szName[MAX_PATH],szFileName[MAX_PATH];
				int Index;
				CColorScheme *pColorScheme;
				LPCTSTR pszFileName;

				GetDlgItemText(hDlg,IDC_COLORSCHEME_PRESET,szName,lengthof(szName));
				if (szName[0]=='\0')
					break;
				Index=(int)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_FINDSTRINGEXACT,-1,(LPARAM)szName);
				if (Index<0
						|| (pColorScheme=pThis->m_PresetList.GetColorScheme(
							(int)SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_GETITEMDATA,Index,0)))==NULL)
					break;
				GetThemesDirectory(szFileName,lengthof(szFileName));
				pszFileName=PathFindFileName(pColorScheme->GetFileName());
				if (lstrlen(szFileName)+1+lstrlen(pszFileName)+lstrlen(m_pszExtension)>=MAX_PATH)
					break;
				PathAppend(szFileName,pszFileName);
				if (pszFileName==szName)
					lstrcat(szFileName,m_pszExtension);
				if (MessageBox(hDlg,TEXT("IꂽzF폜܂?"),TEXT("폜̊mF"),
							   MB_OKCANCEL | MB_ICONQUESTION)!=IDOK)
					break;
				if (!DeleteFile(szFileName)) {
					MessageBox(hDlg,TEXT("t@C폜ł܂B"),NULL,MB_OK | MB_ICONEXCLAMATION);
					break;
				}
				SendDlgItemMessage(hDlg,IDC_COLORSCHEME_PRESET,CB_DELETESTRING,Index,0);
			}
			return TRUE;

		case IDC_COLORSCHEME_LIST:
			switch (HIWORD(wParam)) {
			case LBN_SELCHANGE:
				{
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int SelCount=(int)DlgListBox_GetSelCount(hDlg,IDC_COLORSCHEME_LIST);
					int i;
					COLORREF SelColor=CLR_INVALID,Color;

					if (SelCount==0) {
						pThis->m_ColorPalette.SetSel(-1);
						break;
					}
					if (SelCount==1) {
						for (i=0;i<CColorScheme::NUM_COLORS;i++) {
							if (DlgListBox_GetSel(hDlg,IDC_COLORSCHEME_LIST,i)) {
								SelColor=(COLORREF)DlgListBox_GetItemData(hDlg,IDC_COLORSCHEME_LIST,i);
								break;
							}
						}
					} else {
						for (i=0;i<CColorScheme::NUM_COLORS;i++) {
							if (DlgListBox_GetSel(hDlg,IDC_COLORSCHEME_LIST,i)) {
								Color=(COLORREF)DlgListBox_GetItemData(hDlg,IDC_COLORSCHEME_LIST,i);
								if (SelColor==CLR_INVALID)
									SelColor=Color;
								else if (Color!=SelColor)
									break;
							}
						}
						if (i<CColorScheme::NUM_COLORS) {
							pThis->m_ColorPalette.SetSel(-1);
							break;
						}
					}
					if (SelColor!=CLR_INVALID)
						pThis->m_ColorPalette.SetSel(
									pThis->m_ColorPalette.FindColor(SelColor));
				}
				break;

			case LBN_EX_RBUTTONDOWN:
				{
					CColorSchemeOptions *pThis=GetThis(hDlg);
					HMENU hmenu=::LoadMenu(GetAppClass().GetResourceInstance(),MAKEINTRESOURCE(IDM_COLORSCHEME));
					POINT pt;

					::EnableMenuItem(hmenu,IDC_COLORSCHEME_SELECTSAMECOLOR,
						MF_BYCOMMAND | (pThis->m_ColorPalette.GetSel()>=0?MFS_ENABLED:MFS_GRAYED));
					if (DlgListBox_GetSelCount(hDlg,IDC_COLORSCHEME_LIST)==1) {
						int Sel,Gradient,Border;

						DlgListBox_GetSelItems(hDlg,IDC_COLORSCHEME_LIST,&Sel,1);
						Gradient=CColorScheme::GetColorGradient(Sel);
						if (Gradient>=0) {
							::EnableMenuItem(::GetSubMenu(hmenu,0),2,MF_BYPOSITION | MFS_ENABLED);
							::CheckMenuRadioItem(hmenu,
												 IDC_COLORSCHEME_GRADIENT_NORMAL,IDC_COLORSCHEME_GRADIENT_INTERLACED,
												 IDC_COLORSCHEME_GRADIENT_NORMAL+(int)pThis->m_GradientList[Gradient].Type,
												 MF_BYCOMMAND);
							::EnableMenuItem(::GetSubMenu(hmenu,0),3,MF_BYPOSITION | MFS_ENABLED);
							::CheckMenuRadioItem(hmenu,
												 IDC_COLORSCHEME_DIRECTION_HORZ,IDC_COLORSCHEME_DIRECTION_VERTMIRROR,
												 IDC_COLORSCHEME_DIRECTION_HORZ+(int)pThis->m_GradientList[Gradient].Direction,
												 MF_BYCOMMAND);
							if (!CColorScheme::IsGradientDirectionEnabled(Gradient)) {
								if (pThis->m_GradientList[Gradient].Direction==Theme::DIRECTION_HORZ
										|| pThis->m_GradientList[Gradient].Direction==Theme::DIRECTION_HORZMIRROR) {
									::EnableMenuItem(hmenu,IDC_COLORSCHEME_DIRECTION_VERT,MF_BYCOMMAND | MFS_GRAYED);
									::EnableMenuItem(hmenu,IDC_COLORSCHEME_DIRECTION_VERTMIRROR,MF_BYCOMMAND | MFS_GRAYED);
								} else {
									::EnableMenuItem(hmenu,IDC_COLORSCHEME_DIRECTION_HORZ,MF_BYCOMMAND | MFS_GRAYED);
									::EnableMenuItem(hmenu,IDC_COLORSCHEME_DIRECTION_HORZMIRROR,MF_BYCOMMAND | MFS_GRAYED);
								}
							}
						}
						Border=CColorScheme::GetColorBorder(Sel);
						if (Border>=0) {
							::EnableMenuItem(::GetSubMenu(hmenu,0),4,MF_BYPOSITION | MFS_ENABLED);
							::EnableMenuItem(hmenu,IDC_COLORSCHEME_BORDER_NONE,
											 MF_BYCOMMAND | (CColorScheme::IsBorderAlways(Border)?MFS_GRAYED:MFS_ENABLED));
							::CheckMenuRadioItem(hmenu,
												 IDC_COLORSCHEME_BORDER_NONE,IDC_COLORSCHEME_BORDER_RAISED,
												 IDC_COLORSCHEME_BORDER_NONE+(int)pThis->m_BorderList[Border],
												 MF_BYCOMMAND);
						}
					}
					::GetCursorPos(&pt);
					::TrackPopupMenu(::GetSubMenu(hmenu,0),TPM_RIGHTBUTTON,pt.x,pt.y,0,hDlg,NULL);
					::DestroyMenu(hmenu);
				}
				break;
			}
			return TRUE;

		case IDC_COLORSCHEME_PALETTE:
			switch (HIWORD(wParam)) {
			case CColorPalette::NOTIFY_SELCHANGE:
				{
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int Sel=pThis->m_ColorPalette.GetSel();
					COLORREF Color=pThis->m_ColorPalette.GetColor(Sel);
					int i;

					for (i=0;i<CColorScheme::NUM_COLORS;i++) {
						if (DlgListBox_GetSel(hDlg,IDC_COLORSCHEME_LIST,i))
							DlgListBox_SetItemData(hDlg,IDC_COLORSCHEME_LIST,i,Color);
					}
					InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);
				}
				break;

			case CColorPalette::NOTIFY_DOUBLECLICK:
				{
					CColorSchemeOptions *pThis=GetThis(hDlg);
					int Sel=pThis->m_ColorPalette.GetSel();
					COLORREF Color=pThis->m_ColorPalette.GetColor(Sel);

					if (ChooseColorDialog(hDlg,&Color)) {
						pThis->m_ColorPalette.SetColor(Sel,Color);
						int i;

						for (i=0;i<CColorScheme::NUM_COLORS;i++) {
							if (DlgListBox_GetSel(hDlg,IDC_COLORSCHEME_LIST,i))
								DlgListBox_SetItemData(hDlg,IDC_COLORSCHEME_LIST,i,Color);
						}
						InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);
					}
				}
				break;
			}
			return TRUE;

		case IDC_COLORSCHEME_DEFAULT:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);
				int i;

				for (i=0;i<CColorScheme::NUM_COLORS;i++)
					DlgListBox_SetItemData(hDlg,IDC_COLORSCHEME_LIST,i,
										   CColorScheme::GetDefaultColor(i));
				for (i=0;i<CColorScheme::NUM_GRADIENTS;i++)
					CColorScheme::GetDefaultGradientStyle(i,&pThis->m_GradientList[i]);
				for (i=0;i<CColorScheme::NUM_BORDERS;i++)
					pThis->m_BorderList[i]=CColorScheme::GetDefaultBorderType(i);
				InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);
				::SendMessage(hDlg,WM_COMMAND,IDC_COLORSCHEME_PREVIEW,0);
			}
			return TRUE;

		case IDC_COLORSCHEME_PREVIEW:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				if (pThis->m_pPreviewColorScheme==NULL)
					pThis->m_pPreviewColorScheme=new CColorScheme;
				pThis->GetCurrentSettings(pThis->m_pPreviewColorScheme);
				pThis->Apply(pThis->m_pPreviewColorScheme);
			}
			return TRUE;

		case IDC_COLORSCHEME_SELECTSAMECOLOR:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);
				int Sel=pThis->m_ColorPalette.GetSel();

				if (Sel>=0) {
					COLORREF Color=pThis->m_ColorPalette.GetColor(Sel);
					int TopIndex=(int)DlgListBox_GetTopIndex(hDlg,IDC_COLORSCHEME_LIST);

					::SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,WM_SETREDRAW,FALSE,0);
					for (int i=0;i<CColorScheme::NUM_COLORS;i++) {
						DlgListBox_SetSel(hDlg,IDC_COLORSCHEME_LIST,i,
							(COLORREF)DlgListBox_GetItemData(hDlg,IDC_COLORSCHEME_LIST,i)==Color);
					}
					DlgListBox_SetTopIndex(hDlg,IDC_COLORSCHEME_LIST,TopIndex);
					::SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,WM_SETREDRAW,TRUE,0);
					::InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST);
				}
			}
			return TRUE;

		case IDC_COLORSCHEME_GRADIENT_NORMAL:
		case IDC_COLORSCHEME_GRADIENT_GLOSSY:
		case IDC_COLORSCHEME_GRADIENT_INTERLACED:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				if (DlgListBox_GetSelCount(hDlg,IDC_COLORSCHEME_LIST)==1) {
					int Sel,Gradient;

					DlgListBox_GetSelItems(hDlg,IDC_COLORSCHEME_LIST,&Sel,1);
					Gradient=CColorScheme::GetColorGradient(Sel);
					if (Gradient>=0) {
						pThis->m_GradientList[Gradient].Type=
							(Theme::GradientType)(LOWORD(wParam)-IDC_COLORSCHEME_GRADIENT_NORMAL);
					}
				}
			}
			return TRUE;

		case IDC_COLORSCHEME_DIRECTION_HORZ:
		case IDC_COLORSCHEME_DIRECTION_VERT:
		case IDC_COLORSCHEME_DIRECTION_HORZMIRROR:
		case IDC_COLORSCHEME_DIRECTION_VERTMIRROR:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				if (DlgListBox_GetSelCount(hDlg,IDC_COLORSCHEME_LIST)==1) {
					int Sel,Gradient;

					DlgListBox_GetSelItems(hDlg,IDC_COLORSCHEME_LIST,&Sel,1);
					Gradient=CColorScheme::GetColorGradient(Sel);
					if (Gradient>=0) {
						pThis->m_GradientList[Gradient].Direction=
							(Theme::GradientDirection)(LOWORD(wParam)-IDC_COLORSCHEME_DIRECTION_HORZ);
					}
				}
			}
			return TRUE;

		case IDC_COLORSCHEME_BORDER_NONE:
		case IDC_COLORSCHEME_BORDER_SOLID:
		case IDC_COLORSCHEME_BORDER_SUNKEN:
		case IDC_COLORSCHEME_BORDER_RAISED:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				if (DlgListBox_GetSelCount(hDlg,IDC_COLORSCHEME_LIST)==1) {
					int Sel,Border;

					DlgListBox_GetSelItems(hDlg,IDC_COLORSCHEME_LIST,&Sel,1);
					Border=CColorScheme::GetColorBorder(Sel);
					if (Border>=0) {
						pThis->m_BorderList[Border]=
							(Theme::BorderType)(LOWORD(wParam)-IDC_COLORSCHEME_BORDER_NONE);
						RECT rc;
						::SendDlgItemMessage(hDlg,IDC_COLORSCHEME_LIST,LB_GETITEMRECT,
											 Sel,reinterpret_cast<LPARAM>(&rc));
						InvalidateDlgItem(hDlg,IDC_COLORSCHEME_LIST,&rc);
					}
				}
			}
			return TRUE;
		}
		return TRUE;

	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case PSN_APPLY:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				pThis->GetCurrentSettings(pThis->m_pColorScheme);
				pThis->Apply(pThis->m_pColorScheme);
			}
			break;

		case PSN_RESET:
			{
				CColorSchemeOptions *pThis=GetThis(hDlg);

				if (pThis->m_pPreviewColorScheme!=NULL)
					pThis->Apply(pThis->m_pColorScheme);
			}
			break;
		}
		break;

	case WM_DESTROY:
		{
			CColorSchemeOptions *pThis=GetThis(hDlg);

			if (pThis->m_pPreviewColorScheme!=NULL) {
				delete pThis->m_pPreviewColorScheme;
				pThis->m_pPreviewColorScheme=NULL;
			}
			pThis->m_PresetList.Clear();
			pThis->OnDestroy();
		}
		break;
	}
	return FALSE;
}
